#include "../../src/Battling/Models/Battle.hpp"
#include "../Core.hpp"
using namespace CreatureLib::Battling;

export_func u8 CreatureLib_Battle_Construct(Battle*& out, const BattleLibrary* library, BattleParty* partyArr[],
                                            size_t numberOfParties, bool canFlee, u8 numberOfSides, u8 creaturesPerSide,
                                            uint_fast32_t randomSeed) {
    Try(ArbUt::List<BattleParty*> parties(partyArr, partyArr + numberOfParties);
        out = new Battle(library, parties, canFlee, numberOfSides, creaturesPerSide, randomSeed);)
}

export_func void CreatureLib_Battle_Destruct(const Battle* p) { delete p; }

export_func const BattleLibrary* CreatureLib_Battle_GetLibrary(const Battle* p) { return p->GetLibrary().GetRaw(); }
export_func u8 CreatureLib_Battle_CanUse(bool& out, Battle* p, BaseTurnChoice* turnChoice) {
    Try(out = p->CanUse(turnChoice);)
}
export_func u8 CreatureLib_Battle_TrySetChoice(bool& out, Battle* p, BaseTurnChoice* turnChoice) {
    Try(out = p->TrySetChoice(turnChoice);)
}
export_func bool CreatureLib_Battle_CanFlee(const Battle* p) { return p->CanFlee(); }
export_func u8 CreatureLib_Battle_CheckChoicesSetAndRun(Battle* p) { Try(p->CheckChoicesSetAndRun();) }
export_func u32 CreatureLib_Battle_GetCurrentTurn(Battle* p) { return p->GetCurrentTurn(); }
export_func u32 CreatureLib_Battle_GetCreaturesPerSide(Battle* p) { return p->GetCreaturesPerSide(); }

export_func ChoiceQueue* CreatureLib_Battle_GetCurrentTurnQueue(const Battle* p) {
    return p->GetCurrentTurnQueue().GetRaw();
}
export_func BattleRandom* CreatureLib_Battle_GetRandom(Battle* p) { return p->GetRandom(); }
export_func u8 CreatureLib_Battle_CreatureInField(bool& out, const Battle* p, Creature* c) {
    Try(out = p->CreatureInField(c);)
}
export_func u8 CreatureLib_Battle_GetCreature(Creature*& out, const Battle* p, u8 side, u8 target) {
    Try(out = p->GetCreature(side, target).GetValue();)
}

export_func u8 CreatureLib_Battle_ForceRecall(Battle* p, u8 side, u8 target) { Try(p->ForceRecall(side, target);) }
export_func u8 CreatureLib_Battle_SwitchCreature(Battle* p, u8 side, u8 target, Creature* c) {
    Try(p->SwitchCreature(side, target, c);)
}
export_func u8 CreatureLib_Battle_CanSlotBeFilled(bool& out, const Battle* p, u8 side, u8 target) {
    Try(out = p->CanSlotBeFilled(side, target);)
}
export_func u8 CreatureLib_Battle_ValidateBattleState(Battle* p) { Try(p->ValidateBattleState();) }
export_func bool CreatureLib_Battle_HasEnded(const Battle* p) { return p->HasEnded(); }

export_func bool CreatureLib_Battle_HasConclusiveResult(const Battle* p) { return p->GetResult().IsConclusiveResult(); }
export_func bool CreatureLib_Battle_GetWinningSide(const Battle* p) { return p->GetResult().GetWinningSide(); }

export_func size_t CreatureLib_Battle_GetSidesCount(const Battle* p) { return p->GetSides().Count(); }
export_func BattleSide* const* CreatureLib_Battle_GetSides(const Battle* p) { return p->GetSides().RawData(); }

export_func size_t CreatureLib_Battle_GetPartiesCount(const Battle* p) { return p->GetParties().Count(); }
export_func BattleParty* const* CreatureLib_Battle_GetParties(const Battle* p) {
    return reinterpret_cast<BattleParty* const*>(p->GetParties().RawData());
}

export_func BattleScript* CreatureLib_Battle_GetVolatileScript(Battle* p, const char* key) {
    auto v = p->GetVolatileScript(ArbUt::StringView::CalculateHash(key));
    if (!v.HasValue()) {
        return nullptr;
    }
    return v.GetValue();
}
export_func u8 CreatureLib_Battle_AddVolatileScriptByName(Battle* p, const char* key) {
    Try(p->AddVolatileScript(ArbUt::StringView(key));)
}
export_func u8 CreatureLib_Battle_AddVolatileScript(Battle* p, BattleScript* script) {
    Try(p->AddVolatileScript(script);)
}

export_func u8 CreatureLib_Battle_RemoveVolatileScript(Battle* p, const char* key) {
    Try(p->RemoveVolatileScript(ArbUt::StringView::CalculateHash(key));)
}
export_func u8 CreatureLib_Battle_RemoveVolatileScriptWithScript(Battle* p, BattleScript* script) {
    Try(p->RemoveVolatileScript(script);)
}
export_func bool CreatureLib_Battle_HasVolatileScript(Battle* p, const char* key) {
    return p->HasVolatileScript(ArbUt::StringView::CalculateHash(key));
}

export_func u8 CreatureLib_Battle_RegisterEventListener(Battle* p, void (*func)(const EventData*)) {
    Try(p->RegisterEventListener(func);)
}
export_func const HistoryHolder* CreatureLib_Battle_GetHistory(Battle* p) { return p->GetHistory(); }

export_func u8 CreatureLib_Battle_EndBattle(Battle* p) { Try(p->EndBattle()) }
export_func u8 CreatureLib_Battle_ClearBattle(Battle* p) { Try(p->ClearBattle()) }

SIMPLE_GET_FUNC(Battle, GetLastTurnTimeMicroseconds, long);