#include "../../src/Battling/Models/BattleSide.hpp"
#include "../Core.hpp"
using namespace CreatureLib::Battling;

export_func BattleSide* CreatureLib_BattleSide_Construct(u8 index, Battle* battle, u8 creaturesPerSide) {
    return new BattleSide(index, battle, creaturesPerSide);
}

export_func void CreatureLib_BattleSide_Destruct(BattleSide* p) { delete p; }

export_func Battle* CreatureLib_BattleSide_GetBattle(BattleSide* p) { return p->GetBattle(); }
export_func bool CreatureLib_BattleSide_AllChoicesSet(BattleSide* p) { return p->AllChoicesSet(); }

export_func u8 CreatureLib_BattleSide_AllPossibleSlotsFilled(bool& out, BattleSide* p) {
    Try(out = p->AllPossibleSlotsFilled());
}

export_func u8 CreatureLib_BattleSide_SetChoice(BattleSide* p, BaseTurnChoice* choice) { Try(p->SetChoice(choice);) }

export_func void CreatureLib_BattleSide_ResetChoices(BattleSide* p) { p->ResetChoices(); }

export_func u8 CreatureLib_BattleSide_SetCreature(BattleSide* p, Creature* creature, u8 index) {
    Try(p->SetCreature(creature, index));
}

export_func u8 CreatureLib_BattleSide_GetCreature(Creature*& out, BattleSide* p, u8 index) {
    Try(out = p->GetCreature(index).GetValue());
}

export_func u8 CreatureLib_BattleSide_GetSideIndex(BattleSide* p) { return p->GetSideIndex(); }
export_func u8 CreatureLib_BattleSide_GetCreatureIndex(u8& out, BattleSide* p, Creature* c) {
    Try(out = p->GetCreatureIndex(c));
}

export_func u8 CreatureLib_BattleSide_MarkSlotAsUnfillable(BattleSide* p, Creature* c) {
    Try(p->MarkSlotAsUnfillable(c));
}

export_func bool CreatureLib_BattleSide_IsDefeated(BattleSide* p) { return p->IsDefeated(); }
export_func bool CreatureLib_BattleSide_HasFled(BattleSide* p) { return p->HasFled(); }
export_func void CreatureLib_BattleSide_MarkAsFled(BattleSide* p) { p->MarkAsFled(); }

export_func u8 CreatureLib_BattleSide_SwapPositions(u8& out, BattleSide* p, u8 a, u8 b) {
    Try(out = p->SwapPositions(a, b);)
}

export_func BattleScript* CreatureLib_BattleSide_GetVolatileScript(BattleSide* p, const char* key) {
    auto v = p->GetVolatileScript(ArbUt::StringView::CalculateHash(key));
    if (!v.HasValue()) {
        return nullptr;
    }
    return v.GetValue();
}
export_func u8 CreatureLib_BattleSide_AddVolatileScriptByName(BattleSide* p, const char* key) {
    Try(p->AddVolatileScript(ArbUt::StringView(key));)
}
export_func u8 CreatureLib_BattleSide_AddVolatileScript(BattleSide* p, BattleScript* script) {
    Try(p->AddVolatileScript(script);)
}

export_func u8 CreatureLib_BattleSide_RemoveVolatileScript(BattleSide* p, const char* key) {
    Try(p->RemoveVolatileScript(ArbUt::StringView::CalculateHash(key));)
}
export_func u8 CreatureLib_BattleSide_RemoveVolatileScriptWithScript(BattleSide* p, BattleScript* script) {
    Try(p->RemoveVolatileScript(script);)
}
export_func bool CreatureLib_BattleSide_HasVolatileScript(BattleSide* p, const char* key) {
    return p->HasVolatileScript(ArbUt::StringView::CalculateHash(key));
}
