#include "../../src/Battling/TurnChoices/AttackTurnChoice.hpp"
#include "../../src/Battling/TurnChoices/FleeTurnChoice.hpp"
#include "../../src/Battling/TurnChoices/ItemTurnChoice.hpp"
#include "../../src/Battling/TurnChoices/PassTurnChoice.hpp"
#include "../../src/Battling/TurnChoices/SwitchTurnChoice.hpp"
#include "../Core.hpp"
using namespace CreatureLib::Battling;

export_func AttackTurnChoice* CreatureLib_AttackTurnChoice_Construct(Creature* user, LearnedAttack* attack,
                                                                     u8 sideIndex, u8 targetIndex) {
    return new AttackTurnChoice(user, attack, CreatureIndex(sideIndex, targetIndex));
}
export_func void CreatureLib_AttackTurnChoice_Destruct(AttackTurnChoice* p) { delete p; }
export_func FleeTurnChoice* CreatureLib_FleeTurnChoice_Construct(Creature* user) { return new FleeTurnChoice(user); }
export_func void CreatureLib_FleeTurnChoice_Destruct(FleeTurnChoice* p) { delete p; }
export_func PassTurnChoice* CreatureLib_PassTurnChoice_Construct(Creature* user) { return new PassTurnChoice(user); }
export_func void CreatureLib_PassTurnChoice_Destruct(PassTurnChoice* p) { delete p; }
export_func SwitchTurnChoice* CreatureLib_SwitchTurnChoice_Construct(Creature* user, Creature* newCreature) {
    return new SwitchTurnChoice(user, newCreature);
}
export_func void CreatureLib_SwitchTurnChoice_Destruct(SwitchTurnChoice* p) { delete p; }
export_func ItemTurnChoice* CreatureLib_ItemTurnChoice_ConstructWithoutTarget(Creature* user,
                                                                              CreatureLib::Library::Item* item) {
    return new ItemTurnChoice(user, item, {});
}
export_func ItemTurnChoice* CreatureLib_ItemTurnChoice_ConstructWithTarget(Creature* user,
                                                                           const CreatureLib::Library::Item* item,
                                                                           u8 targetSide, u8 targetIndex) {
    auto index = std::optional(CreatureIndex(targetSide, targetIndex));
    return new ItemTurnChoice(user, item, index);
}
export_func void CreatureLib_ItemTurnChoice_Destruct(SwitchTurnChoice* p) { delete p; }

SIMPLE_GET_FUNC(BaseTurnChoice, GetKind, TurnChoiceKind)
BORROWED_GET_FUNC(BaseTurnChoice, GetUser, Creature*)

BORROWED_GET_FUNC(AttackTurnChoice, GetAttack, LearnedAttack*)
SIMPLE_GET_FUNC(AttackTurnChoice, GetKind, TurnChoiceKind)

export_func u8 CreatureLib_AttackTurnChoice_GetPriority(i8& out, AttackTurnChoice* p) { Try(out = p->GetPriority()); }

OPTIONAL_GET_FUNC(AttackTurnChoice, GetAttackScript, BattleScript*)
export_func u8 CreatureLib_AttackTurnChoice_GetTargetSideIndex(const AttackTurnChoice* p) {
    return p->GetTarget().GetSideIndex();
}
export_func u8 CreatureLib_AttackTurnChoice_GetTargetCreatureIndex(const AttackTurnChoice* p) {
    return p->GetTarget().GetCreatureIndex();
}

OPTIONAL_GET_FUNC(SwitchTurnChoice, GetNewCreature, Creature*)

SIMPLE_GET_FUNC(ItemTurnChoice, GetKind, TurnChoiceKind)
BORROWED_GET_FUNC(ItemTurnChoice, GetItem, const CreatureLib::Library::Item*)
