#include "BattleStatCalculator.hpp"
#include "../Models/Creature.hpp"

using namespace CreatureLib;

Library::StatisticSet<u32>
Battling::BattleStatCalculator::CalculateFlatStats(Battling::Creature* non_null creature) const {
    return Library::StatisticSet<u32>(CalculateFlatStat(creature, Library::Statistic::Health),
                                      CalculateFlatStat(creature, Library::Statistic::PhysicalAttack),
                                      CalculateFlatStat(creature, Library::Statistic::PhysicalDefense),
                                      CalculateFlatStat(creature, Library::Statistic::MagicalAttack),
                                      CalculateFlatStat(creature, Library::Statistic::MagicalDefense),
                                      CalculateFlatStat(creature, Library::Statistic::Speed));
}

Library::StatisticSet<u32>
Battling::BattleStatCalculator::CalculateBoostedStats(Battling::Creature* non_null creature) const {
    return Library::StatisticSet<u32>(CalculateBoostedStat(creature, Library::Statistic::Health),
                                      CalculateBoostedStat(creature, Library::Statistic::PhysicalAttack),
                                      CalculateBoostedStat(creature, Library::Statistic::PhysicalDefense),
                                      CalculateBoostedStat(creature, Library::Statistic::MagicalAttack),
                                      CalculateBoostedStat(creature, Library::Statistic::MagicalDefense),
                                      CalculateBoostedStat(creature, Library::Statistic::Speed));
}

u32 CalculateHealthStat(Battling::Creature* non_null creature) {
    EnsureNotNull(creature)
    auto level = creature->GetLevel();
    float a = (creature->GetBaseStat(Library::Statistic::Health)) * 2.0 * level;
    return static_cast<u32>(floor(a / 100.0) + level + 10);
}

u32 CalculateOtherStat(Battling::Creature* non_null creature, Library::Statistic stat) {
    EnsureNotNull(creature)
    auto level = creature->GetLevel();
    float a = (creature->GetBaseStat(stat)) * 2.0 * level;
    return static_cast<u32>(floor(a / 100.0) + 5);
}

u32 Battling::BattleStatCalculator::CalculateFlatStat(Battling::Creature* non_null creature,
                                                      Library::Statistic stat) const {
    EnsureNotNull(creature);
    if (stat == Library::Statistic::Health)
        return CalculateHealthStat(creature);
    return CalculateOtherStat(creature, stat);
}

u32 Battling::BattleStatCalculator::CalculateBoostedStat(Battling::Creature* non_null creature,
                                                         Library::Statistic stat) const {
    return creature->GetFlatStat(stat) + (creature->GetBoostedStat(stat));
}
