#include "MiscLibrary.hpp"
#include "../Models/Battle.hpp"
#include "../TurnChoices/AttackTurnChoice.hpp"

bool CreatureLib::Battling::MiscLibrary::IsCritical(
    [[maybe_unused]] CreatureLib::Battling::ExecutingAttack* non_null attack,
    CreatureLib::Battling::Creature* non_null target, [[maybe_unused]] u8 hit) const {
    EnsureNotNull(target)
    Ensure(target->GetBattle().HasValue())
    auto rand = target->GetBattle().GetValue()->GetRandom();
    return rand->Get(10) <= 0;
}

static CreatureLib::Battling::LearnedAttack* nullable _replacementAttack = nullptr;
static std::unique_ptr<const CreatureLib::Library::AttackData> _replacementAttackData = nullptr;

static ArbUt::BorrowedPtr<const CreatureLib::Library::AttackData> GetReplacementAttackData() {
    if (_replacementAttackData == nullptr) {
        _replacementAttackData =
            std::unique_ptr<const CreatureLib::Library::AttackData>(new CreatureLib::Library::AttackData(
                "replacement"_cnc, 0, CreatureLib::Library::AttackCategory::Physical, 30, 255, 255,
                CreatureLib::Library::AttackTarget::Any, 0, new CreatureLib::Library::SecondaryEffect(), {}));
    }
    return _replacementAttackData;
}

static CreatureLib::Battling::LearnedAttack* non_null GetReplacementAttack() {
    if (_replacementAttack == nullptr) {
        _replacementAttack = new CreatureLib::Battling::LearnedAttack(
            GetReplacementAttackData(), CreatureLib::Battling::AttackLearnMethod::Unknown);
    }
    return _replacementAttack;
}

bool CreatureLib::Battling::MiscLibrary::CanFlee([[maybe_unused]] FleeTurnChoice* non_null switchChoice) const {
    return true;
}
CreatureLib::Battling::BaseTurnChoice*
CreatureLib::Battling::MiscLibrary::ReplacementAttack(Creature* non_null user,
                                                      [[maybe_unused]] CreatureIndex target) const {
    EnsureNotNull(user)
    Ensure(user->GetBattleSide().HasValue())
    auto sideTarget = 0;
    if (user->GetBattleSide().GetValue()->GetSideIndex() == 0)
        sideTarget = 1;
    auto side = user->GetBattle().GetValue()->GetSides()[sideTarget];
    auto random_mon = side->GetRandomCreatureIndex();
    return new AttackTurnChoice(user, GetReplacementAttack(), CreatureIndex(side->GetSideIndex(), random_mon));
}
